"""Unit testing of iNels switch library."""
from pyinels.const import ATTR_METER

from tests.const_test import (
    TEST_API_CLASS_NAMESPACE,
    TEST_API_ROOM_DEVICES,
    TEST_API_READ_DATA,
    TEST_HOST,
    TEST_PORT,
    TEST_RAW_METER,
    TEST_VERSION,
)

from pyinels.api import Api
from pyinels.device.pyMeter import pyMeter

from unittest.mock import patch
from unittest import TestCase

HUMIDITY_METER_ID = "Humidity_cellar"
HUMIDITY_METER_NAME = "Humidity cellar"
HUMIDITY_METER_UNITS = "Percentage"
HUMIDITY_METER_RETURN_VALUE = {HUMIDITY_METER_ID: 48.56}


class PyMeterTest(TestCase):
    """Class to test iNels meter library."""

    def setUp(self):
        """Setup all necesary instances and mocks."""
        self.patches = [
            patch(f'{TEST_API_CLASS_NAMESPACE}.ping', return_value=True),
            patch(f'{TEST_API_CLASS_NAMESPACE}.{TEST_API_ROOM_DEVICES}',
                  return_value=TEST_RAW_METER),
            patch(f'{TEST_API_CLASS_NAMESPACE}.{TEST_API_READ_DATA}',
                  return_value=HUMIDITY_METER_RETURN_VALUE),
        ]

        for p in self.patches:
            p.start()

        self.api = Api(TEST_HOST, TEST_PORT, TEST_VERSION)
        """Setup all neccessary async stuff."""
        meters = [device for device in self.api.getRoomDevices(
            'garage') if device.type == ATTR_METER]

        self.meter = pyMeter(meters[0])

    def tearDown(self):
        """Destroy all instances and mocks."""
        self.api = None
        self.meter = None
        patch.stopall()
        self.patches = None

    def test_unique_id_and_name_presented(self):
        """Test when the unique id is presented."""
        s = self.meter

        self.assertIsNotNone(s.unique_id)
        self.assertIsNotNone(s.name)

        self.assertEqual(s.unique_id, HUMIDITY_METER_ID)
        self.assertEqual(s.name, HUMIDITY_METER_NAME)
        self.assertEqual(s.units, HUMIDITY_METER_UNITS)



"""Usaage examples."""
from pyinels.api import Api

# fill the IP address and Port of your iNels PLC
api = Api('http://localhost', '9000', 'CU3')

# checking when the connection is established successfuly
available = api.ping()

# get the list of all rooms from Connection server
rooms = api.getRooms()

# get all devices from current room. Room can be taken from rooms property
roomDevices = api.getRoomDevices('room name')

# get the raw data from the PLC formated in string. Each device is on
# separated line it's just used for developing and data checking purposes
raw = api.getRoomDevicesRaw('room name')

# get all devices from all rooms in one list. List contains
# object InelsDevice. This is the most used method.
devices = api.getAllDevices()

# get the value / state of the device. Some special devices like SHUTTERS or
# THERM does not have id. You need to use up/down/temp_current/temp_set as
# param. Method is used for getting the state. So this method will be
# the most offen called one
data = api.observe('InelsDevice.id')

# Write state/data to the PLC.
# You need to change the value of the object device to what ever you want
api.write('device')



"""Const of iNels BUS."""

# Inels attributes
ATTR_ADDITION = "Koef_add"
ATTR_DECIMAL_DIGITS = "Decimal_digits"
ATTR_DOWN = "Down"
ATTR_GROUP = "Group"
ATTR_ID = "Id"
ATTR_MAX_DISP = "Max_disp"
ATTR_MIN_DISP = "Min_disp"
ATTR_MULTIPLICATOR = "Koef_mult"
ATTR_READ_ONLY = "Read_only"
ATTR_RELE = "Rele"
ATTR_STOP = "Stop"
ATTR_SWITCH_OFF = 0
ATTR_SWITCH_ON = 1
ATTR_TEMP = "Temp"
ATTR_TEMP_SET = "Temp_set"
ATTR_TITLE = "Title"
ATTR_TYPE = "Type"
ATTR_UNITS = "Units"
ATTR_UP = "Up"


# Internal device attributes
ATTR_AIRING = "airing"
ATTR_DOOR = "door"
ATTR_HEATING = "heating"
ATTR_LIGHT = "light"
ATTR_METER = "meter"
ATTR_SCENE = "scenes"
ATTR_SHUTTER = "shutter"
ATTR_SWITCH = "switch"
ATTR_THERM = "therm"
ATTR_UNKNOWN = "unknown"

# Device types dictionary with all determined typs translated to better
# understand
DEVICE_TYPE_DICT = {
    "airing": "airing",
    "garage": "door",
    "gate": "door",
    "hc4": "unknown",
    "heating": "heating",
    "heat-control": "therm",
    "lights": "light",
    "meter": "meter",
    "on_off": "switch",
    "scenes": "scenes",
    "shutters": "shutter",
    "thermals": "thermals",
    "undefined": "undefined",
}

DIRECTIONS_DICT = {
    "Down": "down",
    "Stop": "stop",
    "Up": "up",
}

INELS_BUS_ATTR_DICT = {
    "Decimal_digits": "decimal_digits",
    "Down": "down",
    "Group": "group",
    "Id": "inels",
    "Koef_add": "koef_add",
    "Koef_mult": "koef_mult",
    "Max_disp": "max_disp",
    "Min_disp": "min_disp",
    "Read_only": "read_only",
    "Rele": "rele",
    "Temp": "therm",
    "Temp_set": "stateth",
    "Title": "name",
    "Type": "type",
    "Units": "units",
    "Up": "up"
}

UNITS_DICT = {
    "stu": "Degree",
    "%": "Percentage"
}

ERROR_VALUE = -1

NAME = "pyinels"

RANGE_BLIND = (0, 100)
RANGE_BRIGHTNESS = (0.0, 100.0)

SUPPORT_BRIGHTNESS = 1
SUPPORT_RGB_COLOR = 2
SUPPORT_COLOR_TEMP = 4
SUPPORT_HEX_COLOR = 8

# supported features
SUPPORT_OPEN = 1
SUPPORT_CLOSE = 2
SUPPORT_SET_POSITION = 4,
SUPPORT_STOP = 8
SUPPORT_OPEN_TILT = 16
SUPPORT_CLOSE_TILT = 32
SUPPORT_STOP_TILT = 64

STATE_CLOSED = "closed"
STATE_CLOSING = "closing"
STATE_OPEN = "open"
STATE_OPENING = "opening"

VERSION = "0.6.15"



"""Unit testing of iNels real library."""

from pyinels.api import Api
from unittest import TestCase

# from pyinels.const import ATTR_LIGHT, ATTR_SHUTTER
# from pyinels.device.pyLight import pyLight
# from pyinels.device.pyShutter import pyShutter


class PyRealTests(TestCase):
    """Class to test iNels real connection."""

    def setUp(self):
        self.api = Api("http://192.168.2.102", 8001, "CU3")

    def tearDown(self):
        """Destroy all instances."""
        self.api = None

    # def test_online(self):
    #     """Test in real world"""

    #     ping = self.api.ping()
    #     self.assertTrue(ping)

    #     devices = self.api.getAllDevices()
    #     self.assertGreater(len(devices), 0)

    #     lights = [device for device in devices if device.type == ATTR_LIGHT]
    #     self.assertGreater(len(lights), 0)

    #     light = pyLight(lights[6])
    #     self.assertEqual(light.name, "Pokoj")

    #     light.set_brightness(20)

    # def test_devices_data_fetch(self):
        # """Fetching all data from devices."""

        # devices = self.api.getAllDevices()
        # self.assertGreater(len(devices), 0)

        # persisted_devices = self.api.devices
        # self.assertGreater(len(persisted_devices), 0)

        # lights = [device for device in devices if device.type == ATTR_LIGHT]
        # self.assertGreater(len(lights), 0)
        # light = pyLight(lights[6])
        # self.assertEqual(light.name, "Pokoj")

        # light.set_brightness(20)

        # devs_values = self.api.read_all()
        # self.assertGreater(len(devs_values), 0)

        # l_value = light._device.get_value()
        # self.assertEqual(l_value[light.unique_id], 0)

    # def test_shutter_data_fetch(self):
    #     """Fetching shutter data from api."""

    #     devices = self.api.getAllDevices()
    #     self.assertGreater(len(devices), 0)

    #     shutters = [device for device in devices
    # if device.type == ATTR_SHUTTER]
    #     self.assertGreater(len(shutters), 0)

    #     shutter = pyShutter(shutters[0])
    #     self.assertEqual(shutter.name, "Zal OB")

    #     value = shutter._device.get_value()
    #     self.assertGreater(len(value), 0)



"""Unit testing of iNels airing library."""
from pyinels.const import ATTR_AIRING

from tests.const_test import (
    TEST_API_CLASS_NAMESPACE,
    TEST_API_ROOM_DEVICES,
    TEST_API_READ_DATA,
    TEST_HOST,
    TEST_PORT,
    TEST_RAW_AIRING,
    TEST_VERSION
)

from pyinels.api import Api
from pyinels.device.pyAiring import pyAiring

from unittest.mock import patch
from unittest import TestCase

AIRING_ID = "RE12_VT_equipment_room"
AIRING_NAME = "Equipment_room"

AIRING_RETURN_OFF = {AIRING_ID: 0}
AIRING_RETURN_ON = {AIRING_ID: 1}


class PyAiringTest(TestCase):
    """Class to test iNels airing library."""

    def setUp(self):
        """Setup all necesary instances and mocks."""
        self.patches = [
            patch(f'{TEST_API_CLASS_NAMESPACE}.ping', return_value=True),
            patch(f'{TEST_API_CLASS_NAMESPACE}.{TEST_API_ROOM_DEVICES}',
                  return_value=TEST_RAW_AIRING),
            patch(f'{TEST_API_CLASS_NAMESPACE}.{TEST_API_READ_DATA}',
                  return_value=AIRING_RETURN_OFF),
            patch(f'{TEST_API_CLASS_NAMESPACE}._Api__writeValues',
                  return_value=None)
        ]

        for p in self.patches:
            p.start()

        self.api = Api(TEST_HOST, TEST_PORT, TEST_VERSION)
        """Setup all neccessary async stuff."""
        airings = [device for device in self.api.getRoomDevices(
            'garage') if device.type == ATTR_AIRING]

        self.airing = pyAiring(airings[0])

    def tearDown(self):
        """Destroy all instances and mocks."""
        self.api = None
        self.airing = None
        patch.stopall()
        self.patches = None

    def test_state(self):
        """Test the state of the pyAiring."""
        s = self.airing

        s.update()
        # the airing at the beggining should be turned off
        self.assertFalse(s.state)

    def test_unique_id_and_name_presented(self):
        """Test when the unique id is presented."""
        s = self.airing

        self.assertIsNotNone(s.unique_id)
        self.assertIsNotNone(s.name)

        self.assertEqual(s.unique_id, AIRING_ID)
        self.assertEqual(s.name, AIRING_NAME)

    def test_turn_on(self):
        """Test turn on the airing."""
        s = self.airing

        s.update()
        self.assertFalse(s.state)

        with patch.object(self.api, TEST_API_READ_DATA,
                          return_value=AIRING_RETURN_ON):
            s.turn_on()

            s.update()
            self.assertTrue(s.state)

    def test_turn_off(self):
        """Test turn off the airing."""
        s = self.airing

        with patch.object(self.api, TEST_API_READ_DATA,
                          return_value=AIRING_RETURN_ON):
            s.turn_on()

            s.update()
            self.assertTrue(s.state)

        s.turn_off()

        s.update()
        self.assertFalse(s.state)



"""Inels meter class for iNels BUS."""

from pyinels.device.pyBase import pyBase

from pyinels.const import (
    UNITS_DICT,
    ERROR_VALUE
)


class pyMeter(pyBase):
    """Meter class based on InelsDevice."""

    def __init__(self, device):
        """Initialize object."""
        super().__init__(device)

    @property
    def value(self):
        """Value of the device."""
        val = self._device.value[self._device.id]

        if isinstance(val, str):
            return ERROR_VALUE
        else:
            val = max(val, self._device.min_display_value) \
                if self._device.min_display_value else val
            val = min(val, self._device.max_display_value) \
                if self._device.max_display_value else val
            val = val * self._device.multiplicator \
                if self._device.multiplicator else val
            val = val * self._device.addition \
                if self._device.addition else val
            val = round(val, self._device.decimal_digits) \
                if self._device.decimal_digits else val
            return val

    @property
    def units(self):
        """Units of the device."""
        if self._device.units not in UNITS_DICT:
            return None
        return UNITS_DICT.get(self._device.units)



"""Inels airing class for iNels BUS."""

from pyinels.device.pyBase import pyBase


class pyAiring(pyBase):
    """Airing class based on InelsDevice."""

    def __init__(self, device):
        """Initialize pyAiring class."""
        super().__init__(device)
